#! /usr/bin/python
# encoding=utf-8
# Created by Fenglu Niu on 2020/9/2 16:05

from fengluU.mysql_helper import *


def gen_alter(table: str = None):
    alter_sqls_temp = []
    fields_old = helper_old.get_table_fields(table=table)
    fields_new = helper_new.get_table_fields(table=table)
    alter_sqls_temp.extend(alter(table, fields_old, fields_new))
    alter_sqls_temp.extend(alter_modify(table, fields_old, fields_new))
    return alter_sqls_temp


def alter_add(table, diffs):
    alter_add_sqls = []
    for diff in diffs:
        key = diff.get('key')
        type = diff.get('type')
        null = diff.get('null')
        default = diff.get('default')
        alter_add_sql = f'ALTER TABLE {table} ADD {key} {type}{f" DEFAULT {default}" if default else ""} {null};'
        alter_add_sqls.append(alter_add_sql)
    return alter_add_sqls


def alter_change(table, diffs):
    alter_add_sqls = []
    for diff in diffs:
        old = diff.get('old')
        key = diff.get('key')
        type = diff.get('type')
        null = diff.get('null')
        default = diff.get('default')
        alter_add_sql = f'ALTER TABLE {table} CHANGE {old} {key} {type}{f" DEFAULT {default}" if default else ""} {null};'
        alter_add_sqls.append(alter_add_sql)
    return alter_add_sqls


def interactive(table, diffs, diffs_old):
    keys = [row.get('key') for row in diffs_old]
    diffs_add = []
    diffs_change = []
    print('\033[31m', end='')
    print('\n接下来进入自主确认改名字段:')
    print('\033[0m', end='')
    print('表名：', end='')
    print('\033[1;31m', end='')
    print(f'【{table}】', end='')
    print('\033[0m', end='')
    print(f',待确认字段{keys}')
    for i, diff in enumerate(diffs_old):
        prompt = ', '.join([f'\033[1;31m{i}\033[0m)、{e.get("key")}' for i, e in enumerate(diffs)])
        user_input = int(input(f'[\033[31m{diff.get("key")}\033[0m]要改成哪个呢? {prompt}: '))
        while user_input not in range(len(diffs)):
            user_input = int(input(f'请您不要玩花样，就输入提示中有的数字!{prompt}: '))
        diff_change = diffs.pop(user_input)
        diff_change.setdefault('old', diff.get('key'))
        diffs_change.append(diff_change)
    diffs_add.extend(diffs)
    return diffs_add, diffs_change


def alter(table, fields_old, fields_new):
    alter_sqls = []
    diffs = [f for f in fields_new if f.get('key') not in [f.get('key') for f in fields_old]]
    diffs_old = [f for f in fields_old if f.get('key') not in [f.get('key') for f in fields_new]]
    delta = len(fields_new) - len(fields_old)
    if len(diffs) == delta:
        alter_sqls.extend(alter_add(table, diffs))
    elif len(diffs) == 1:
        diffs[0].setdefault('old', diffs_old[0].get('key'))
        alter_sqls.extend(alter_change(table, diffs))
    else:
        diffs_add, diffs_change = interactive(table, diffs, diffs_old)
        alter_sqls.extend(alter_add(table, diffs_add))
        alter_sqls.extend(alter_change(table, diffs_change))
    return alter_sqls


def alter_modify(table, fields_old, fields_new):
    alter_modify_sqls = []
    diffs = [f for f in fields_new if f.get('key') in [f1.get('key') for f1 in fields_old if
                                                       f1.get('type') != f.get('type') or f1.get('null') != f.get(
                                                           'null') or f1.get('default') != f.get('default')]]
    for diff in diffs:
        key = diff.get('key')
        type = diff.get('type')
        null = diff.get('null')
        default = diff.get('default')
        alter_modify_sql = f'ALTER TABLE {table} MODIFY {key} {type}{f" DEFAULT {default}" if default else ""} {null};'
        alter_modify_sqls.append(alter_modify_sql)
    return alter_modify_sqls


config_old = {
    'user': 'king',
    'password': 'king001',
    'host': '127.0.0.1',
    'database': 'queen',
    'charset': 'utf8',
    'raise_on_warnings': True,
}

config_new = {
    'user': 'king',
    'password': 'king001',
    'host': '127.0.0.1',
    'database': 'king',
    'charset': 'utf8',
    'raise_on_warnings': True,
}

if __name__ == '__main__':
    helper_old = MySQLHelper()
    helper_old.set_config(**config_old)
    helper_new = MySQLHelper()
    helper_new.set_config(**config_new)
    tables = helper_old.show_tables()
    alter_sqls = gen_alter('student')
    print('\n\033[32m生成ALTER SQL如下:\033[0m')
    for sql in alter_sqls:
        print(sql)
    pass



#! /usr/bin/python
# encoding=utf-8
# Created by Fenglu Niu on 2018/06/04 09:35
from fengluU import NFLError

__all__ = ['number2upper', 'append_uint']


class N2U(object):
    __instance = None
    __is_first = True

    def __new__(cls):
        if not cls.__instance:
            cls.__instance = object.__new__(cls)
        return cls.__instance

    def __init__(self):
        if self.__is_first:
            N2U.__is_first = False
            self.__CN_UNIT = ['万', '亿', '兆', '京', '垓', '秭', '穰', '沟', '涧', '正', '载', '极', '归', '无量']
            self.__CN_NUM = ['零', '一', '二', '三', '四', '五', '六', '七', '八', '九']

    def append_uint(self, unit: str):
        self.__CN_UNIT.append(unit)

    def pop_unit(self):
        self.__CN_UNIT.pop()

    def number2upper(self, num: int) -> str:
        """将输入的数字按照中文的书写方式返回(支持的数字长度是65536位数字)
        例如：
            123456789\n
            一亿二千三百四十五万六千七百八十九\n
            算法如下：
                算法采用的是上数法，数穷则变，就是：万万曰亿，亿亿曰兆，兆兆曰京...
                数字从右往左数，可以获取到每个数字的编号，起始编号为0\n
                用该位数字编号和4取模,若有余，余1为十，余2为百，余3为千\n
                若整除，则用该位数字编号与4整除，若结果为奇数，则该位单位为万\n
                若结果为偶数:
                    1: 先判断该结果是否是2的整数次方,若是，该位单位是CN_UNIT[次方]\n
                    2：若不是:从亿位开始作为检测标记位，用结果和(2^检测标记位下标)取余\n
                       若能整除，并且商为奇数则单位是 CN_INIT[标记检测位下标]

            另外，我这个方法支持的数字真的非常大，我可以不判断数字长度吗？
            每添加一个更大的单位在上面的UN_UNIT列表的最后面，您就可以将当前的长度扩充一倍\n
            添加方法：n2u.append_unit('更大的单位')

        :param num: 一个整型数字
        :return: 返回数字的中文书写方式
        """

        def cal_unit(num_index: int) -> str:
            """计算对应编号数字的单位

            :param num_index: 数字的编号，指的是数字中的某一位数字在该数字中从右往左数的编号，编号从0开始\n
            :return: 返回指定编号位数字的单位
            """
            if num_index == 0:  # 如果数字为个位，没有单位
                return ''
            remain = num_index % 4
            if remain > 0:
                if remain == 1:
                    return '十'
                elif remain == 2:
                    return '百'
                elif remain == 3:
                    return '千'
            else:
                consult_4 = num_index // 4
                if consult_4 % 2 != 0:
                    return '万'
                else:
                    bin_consult_4 = bin(consult_4)
                    c1_bin_consult_4 = bin_consult_4.split('b')[1].count('1')
                    c0_bin_consult_4 = bin_consult_4.split('b')[1].count('0')
                    if c1_bin_consult_4 == 1:
                        return self.__CN_UNIT[c0_bin_consult_4]
                    else:
                        for check_index in range(1, len(self.__CN_UNIT)):
                            if consult_4 % (2 ** check_index) == 0:
                                checked_consult_4 = consult_4 // (2 ** check_index)
                                if checked_consult_4 % 2 != 0:
                                    return self.__CN_UNIT[check_index]

        # 首先验证输入的数字是否合法
        if not isinstance(num, int):
            raise NFLError('您输入的数字不是整数吧，我只能支持整数，我在正在努力学习，嘤嘤嘤')
        upper_num = ''
        # 首先将数字转换成单个数字的列表
        num_list = [int(str_num) for str_num in str(num)]
        len_num = len(num_list)
        max_len = 2 ** (len(self.__CN_UNIT) + 2)
        if len_num > max_len:
            raise NFLError('您输入的数字太长了，我只能支持' + str(max_len) + '位数字，嘤嘤嘤')
        for i in range(len_num):
            num_bit = num_list[i]

            i_index = len_num - i - 1
            # 根据算法计算出单位
            unit = cal_unit(i_index)
            # 如果首位数字为1并且为十位，则一可不写

            # 处理该位数字是0的情况
            if num_bit == 0:
                if self.__CN_UNIT.count(unit) == 1:
                    """数字为0,但是该位数字的单位是CN_UNIT中的一个的情况\n
                        如果前一位是零，则删除零
                            这里会有一种极端情况，如果接下的超过四位数都为零，则会出现两个单位碰到一起\n
                            这时候就在末尾用零代替第二个单位
                    """
                    if upper_num.endswith('零'):
                        upper_num = upper_num.rstrip('零')
                        # 默认单位长度为一，处理单位长度大于一的情况
                        stop = -2
                        last_unit = upper_num[-1:stop:-1][::-1]
                        while not ['十', '百', '千'].count(last_unit) and not self.__CN_UNIT.count(last_unit):
                            stop -= 1
                            last_unit = upper_num[-1:stop:-1][::-1]
                        if self.__CN_UNIT.count(last_unit):
                            if self.__CN_UNIT.index(unit) - self.__CN_UNIT.index(last_unit) < 1:
                                upper_num += '零'
                                continue
                    upper_num += unit
                else:  # 如果数字为零并且不在上述单位，并且前一位不是零，则加上零
                    if not upper_num.endswith('零'):
                        upper_num += self.__CN_NUM[num_bit]
            else:  # 数字不是零的情况就加上转换后的大写数字和单位
                if not (unit == '十' and i == 0 and num_bit == 1):  # 若首位数字为1并且该位为十位就不需要一
                    upper_num += self.__CN_NUM[num_bit]
                upper_num += unit
        if len(upper_num) > 1:
            upper_num = upper_num.rstrip('零')
        return upper_num


def number2upper(num: int) -> str:
    """将输入的数字按照中文的书写方式返回(支持的数字长度是65536位数字)\n
        例如：
            123456789\n
            一亿二千三百四十五万六千七百八十九\n

        :param num: 一个整型数字
        :return: 返回数字的中文书写方式

    """
    n2u = N2U()
    return n2u.number2upper(num)


def append_uint(unit: str):
    """向单位列表后添加更大的单位

    :param unit: 添加的更大的单位
    """

    n2u = N2U()
    n2u.append_uint(unit)


def pop_unit():
    """ 删除最后一个单位
    """
    n2u = N2U()
    n2u.pop_unit()


if __name__ == '__main__':
    sn = '1' + '0' * 65535
    append_uint("牛逢路")
    # pop_unit()
    print(number2upper(int(sn)))



#! /usr/bin/python
# encoding=utf-8
# Created by Fenglu Niu on 2018/07/05 10:11
from typing import Any, Tuple

from mysql.connector import pooling
from mysql.connector.errors import IntegrityError
import sys

from mysql.connector.pooling import PooledMySQLConnection

from fengluU import NFLError

from fengluU import n2u

__all__ = ['MySQLHelper', 'QueryDict', 'QueryDictList', 'QueryOpt']


def auto_close(func):
    """
    universal auto_close decorator
    """

    def call_func(self, *args, **kwargs):
        cnx = self.get_cnx()
        ret = func(self, cnx=cnx, *args, **kwargs)
        assert isinstance(cnx, PooledMySQLConnection)
        cnx.close()
        return ret

    return call_func


class QueryOpt(object):
    REL_AND = 'AND'
    REL_OR = 'OR'
    OPT_IS = ' IS '
    OPT_IS_NOT = ' IS NOT '
    OPT_EQUAL = '='
    OPT_GT = '>'
    OPT_GTE = '>='
    OPT_LT = '<'
    OPT_LTE = '<='
    OPT_NE = '<>'
    OPT_LIKE = ' LIKE '


class QueryDict(object):

    def __init__(self, key=None, opt=QueryOpt.OPT_EQUAL, value=None, rel=QueryOpt.REL_AND):
        self.__key = key
        self.__opt = opt
        self.__value = value
        self.__rel = rel

    @property
    def key(self) -> str:
        return self.__key

    @key.setter
    def key(self, key: str) -> None:
        self.__key = key

    @property
    def opt(self) -> str:
        return self.__opt

    @opt.setter
    def opt(self, opt: str) -> None:
        self.__opt = opt

    @property
    def value(self) -> str:
        return self.__value

    @value.setter
    def value(self, value: str) -> None:
        self.__value = value

    @property
    def rel(self) -> str:
        return self.__rel

    @rel.setter
    def rel(self, rel: str) -> None:
        self.__rel = rel

    def put(self, key='', opt=QueryOpt.OPT_EQUAL, value=Any, rel=QueryOpt.REL_AND) -> None:
        self.__key = key
        self.__opt = opt
        self.__value = value
        self.__rel = rel


class QueryDictList(list):
    def __init__(self):
        super().__init__()

    def put(self, qd: QueryDict) -> None:
        self.append(qd)


class MySQLHelper(object):
    """
    MySQLHelper MySQL工具类，便于数据库连接池及连接的管理和增删改查\n
        用法:
            config = {\n
                'user': '****',\n
                'charset': 'utf8',\n
                'password': '****',\n
                'host': 'mysql server host', # 127.0.0.1\n
                'database': '****',\n
                'raise_on_warnings': True,\n
            }\n
            helper = MySQLHelper()\n
            helper.set_config(**config)\n
    """
    __instance = None
    __is_first = True

    # def __new__(cls):
    #     if not cls.__instance:
    #         cls.__instance = object.__new__(cls)
    #     return cls.__instance

    def __init__(self):
        if self.__is_first:
            MySQLHelper.__is_first = False
            self.__logging = False

    def get_cnx(self):
        return self.__cnxpool.get_connection()

    @property
    def logging(self) -> str:
        return self.logging

    @logging.setter
    def logging(self, logging: bool):
        self.__logging = logging

    @staticmethod
    def generate_where(ql: QueryDictList) -> Tuple[str, dict]:
        where = ' WHERE 1=1 '
        if ql:
            params = {qd.key: qd.value for qd in ql}
            where += ''.join([qd.rel + ' ' + qd.key + qd.opt + '%(' + qd.key + ')s ' for qd in ql])
        return where, params

    @staticmethod
    def execute(cursor=None, sql=str, params=tuple) -> str:
        if '%s' in sql:
            if not params:
                raise NFLError("如果使用sql，且sql中有占位符号,则必须传递params参数")
            else:
                cursor.execute(sql, params)
        else:
            cursor.execute(sql)

    @auto_close  # query = auto_close(query)
    def query(self, cnx=None, **kwargs) -> list:
        """

        数据库小助手-查询
        \n:param cnx: 数据连接，装饰器自动赋值
        \n:param kwargs: 可以识别的关键字参数：sql，如果存在并且含有placeholder %s,则必须存在params就直接使用sql进行查询
            如果不存在，那么使用指定了表名和查询字段和条件的查询方式
        \n:return: 将查询的数据以[{}]的形式返回\n
        """
        result = []
        # cnx = self.get_cnx()
        sql = kwargs.get('sql')
        params = kwargs.get('params')
        cursor = cnx.cursor(dictionary=True)
        if sql is not None:
            MySQLHelper.execute(cursor=cursor, sql=sql, params=params)
        else:
            tablename = kwargs.get('tablename')
            ql = kwargs.get('query_list')
            if not tablename:
                raise NFLError('如果没有传sql，必须传tablename')
            sql = 'select * from ' + tablename + ' where 1=1 '
            where, params = MySQLHelper.generate_where(ql)
            cursor.execute(sql + where, params)
        for row in cursor:
            result.append(row)
        return result

    @auto_close
    def query_count(self, cnx=None, **kwargs) -> list:
        """
        
        数据库小助手-查询数据的条数\n
        :param cnx: 数据连接，装饰器自动赋值
        :param kwargs: 可以识别的关键字参数：sql，如果存在，就直接使用sql进行查询
            如果不存在，那么使用指定了表名和查询字段和条件的查询方式
        :return: 将查询的数据以[{}]的形式返回\n
        """
        count = 0
        # cnx = self.get_cnx()
        cursor = cnx.cursor()
        sql = kwargs.get('sql')
        if sql is not None:
            MySQLHelper.execute(cursor=cursor, sql=sql)
        else:
            tablename = kwargs.get('tablename')
            ql = kwargs.get('query_list')
            if not tablename:
                raise NFLError('如果没有传sql，必须传tablename')
            sql = 'select count(*) from ' + tablename + ' where 1=1 '
            where, params = MySQLHelper.generate_where(ql)
            cursor.execute(sql + where, params)
        count = cursor.fetchone()[0]
        # cnx.close()
        return count

    @auto_close
    def insert(self, cnx=None, **kwargs) -> tuple:
        """

        数据库小助手-插入
        \n:param cnx: 数据库连接，装饰器自动赋值
        \n:param kwargs: 可以识别的关键字参数：sql，如果存在，就直接使用sql进行插入操作
            如果不存在，那么使用指定了表名和查询字段和条件的查询方式
        \n:return: 将插入的行数予以返回
        """
        row = 0
        rowid = None
        # cnx = self.get_cnx()
        cursor = cnx.cursor()
        sql = kwargs.get('sql')
        params = kwargs.get('params')
        try:
            if sql is not None:
                MySQLHelper.execute(cursor=cursor, sql=sql, params=params)
            else:
                tablename = kwargs.get('tablename')
                params = {}
                ql = kwargs.get('query_list')
                ql_list = kwargs.get('ql_list')
                if not tablename:
                    raise NFLError('如果没有传sql，必须传tablename')
                if not ql:
                    if not ql_list or not len(ql_list):
                        raise NFLError('插入语句params必须有query_list<QueryDictList>字段,或者ql_list<list<QueryDictList>>字段')

                sql = 'INSERT INTO ' + tablename
                names = ' ('
                values = ' VALUES ('
                if ql:
                    assert isinstance(ql, QueryDictList)
                    names += ','.join([qd.key for qd in ql])
                    values += ','.join(['%(' + qd.key + ')s' for qd in ql])
                    params = {qd.key: qd.value for qd in ql}
                    values += ")"
                elif ql_list:
                    names += ','.join([qd.key for qd in ql_list[0]])
                    values = ' VALUES '
                    values += ',\n'.join(['(' + ', '.join(value) + ')' for value in
                                          [['%(' + qd.key + str(i) + ')s' for qd in ql] for i, ql in
                                           enumerate(ql_list)]])
                    params = {item[0].key + str(item[1]): item[0].value for qdi in
                              [[(qd, i) for qd in ql] for i, ql in enumerate(ql_list)] for item in qdi}

                names += ")"
                print(sql + names + values)
                cursor.execute(sql + names + values, params)
            row = cursor.rowcount
            rowid = cursor.lastrowid
            cnx.commit()
        except IntegrityError as e:
            print(e, file=sys.stderr)
            print(params)
        # cnx.close()
        return row, rowid

    @auto_close
    def update(self, cnx=None, **kwargs) -> int:
        """

        数据库小助手-更新\n
        :param cnx: 数据库连接，装饰器自动赋值
        :param kwargs: 可以识别的关键字参数：sql，如果存在，就直接使用sql进行更新操作
            如果不存在，那么使用指定了表名和更新字段和条件的更新方式
        :return: 将更新的行数予以返回
        """
        row = 0
        # cnx = self.get_cnx()
        cursor = cnx.cursor()
        sql = kwargs.get('sql')
        params = kwargs.get('params')
        try:
            if sql is not None:
                MySQLHelper.execute(cursor=cursor, sql=sql, params=params)
            else:
                tablename = kwargs.get('tablename')
                params = {}
                ul = kwargs.get('update_list')
                ql = kwargs.get('query_list')
                if not tablename:
                    raise NFLError('如果没有传sql，必须传tablename')
                if not ul:
                    raise NFLError('更新语句params必须有update_list<QueryDictList>字段')
                assert isinstance(ul, QueryDictList)
                sql = 'UPDATE ' + tablename + ' SET '
                updates = ', '.join([ud.key + '=%(' + ud.key + ')s' for ud in ul])
                params = {ud.key: ud.value for ud in ul}
                where, ql_params = MySQLHelper.generate_where(ql=ql)
                params.update(ql_params)
                # print(sql + updates + where)
                cursor.execute(sql + updates + where, params)
            row = cursor.rowcount
            cnx.commit()
        except IntegrityError as e:
            print(e, file=sys.stderr)
            print(params)
        # cnx.close()
        return row

    @auto_close
    def delete(self, cnx=None, **kwargs) -> int:
        """
        数据库小助手-删除\n
        :param cnx: 数据库连接，装饰器自动赋值
        :param kwargs: 可以识别的关键字参数：sql，如果存在，就直接使用sql进行更新操作
            如果不存在，那么使用指定了表名和条件的删除方式
        :return: 将删除的行数予以返回
        """
        row = 0
        # cnx = self.get_cnx()
        cursor = cnx.cursor()
        sql = kwargs.get('sql')
        params = kwargs.get('params')
        try:
            if sql is not None:
                MySQLHelper.execute(cursor=cursor, sql=sql, params=params)
            else:
                tablename = kwargs.get('tablename')
                params = {}
                ql = kwargs.get('query_list')
                if not tablename:
                    raise NFLError('如果没有传sql，必须传tablename')
                if not ql:
                    raise NFLError('更新语句params必须有query_list<QueryDictList>字段')
                assert isinstance(ql, QueryDictList)
                sql = 'DELETE FROM ' + tablename
                where, params = MySQLHelper.generate_where(ql=ql)
                # print(sql + where)
                cursor.execute(sql + where, params)
            row = cursor.rowcount
            cnx.commit()
        except IntegrityError as e:
            print(e, file=sys.stderr)
            print(params)
        # cnx.close()
        return row

    @auto_close
    def show_tables(self, cnx: PooledMySQLConnection = None) -> list:
        """
        数据库小助手-获取库中的表\n
        :param cnx: 数据库连接，装饰器自动赋值
        :return: 将数据库中表名列表予以返回
        """
        cursor = cnx.cursor()
        cursor.execute("SHOW TABLES")
        tables = [row[0] for row in cursor.fetchall()]
        return tables

    @auto_close
    def get_table_fields(self, cnx: PooledMySQLConnection = None, table: str = None):
        """
        数据库小助手-获取表结构\n
        :param cnx: 数据库连接，装饰器自动赋值
        :param table: 表名
        :return: 将表结构的字典列表予以返回（key:字段名，type:字段类型，null:是否为空）
        """
        cursor = cnx.cursor()
        cursor.execute(f"DESC {table}")
        feilds = [{'key': f[0], 'type': f[1], 'null': 'NULL' if f[2] == 'YES' else 'NOT NULL'} for f in cursor.fetchall()]
        return feilds

    def set_config(self, **config):
        self.__cnxpool = pooling.MySQLConnectionPool(pool_name='nfl_pool',
                                                     pool_size=32,
                                                     **config
                                                     )


if __name__ == '__main__':
    config = {
        'user': 'king',
        'password': 'king001',
        'host': '127.0.0.1',
        'database': 'king',
        'charset': 'utf8',
        'raise_on_warnings': True,
    }
    helper = MySQLHelper()
    helper.set_config(**config)
    # 启用日志与否
    helper.logging = True

    print(helper.show_tables())
    print(helper.get_table_fields(table='student'))

    # 下面是示例代码
    # Student表结构
    # +-------+-------------+------+-----+---------+----------------+
    # | Field | Type       | Null  | Key | Default | Extra          |
    # +-------+-------------+------+-----+---------+----------------+
    # | id   | int(11)     | NO   | PRI | NULL     | auto_increment |
    # | name | varchar(32) | NO   |     | NULL     |                |
    # | age | int(11)      | YES  |     | NULL     |                |
    # +-------+-------------+------+-----+---------+----------------+

    # 通过SQL查询
    # querykw = {}
    # querykw.setdefault('sql', 'select * from student where id=%s')
    # querykw.setdefault('params', (1,))
    # # querykw.setdefault('tablename', 'areas')
    # rows = helper.query(**querykw)
    # print(rows)

    # # 通过指定表名和条件查询
    # ql = QueryDictList()
    # qd = QueryDict()
    # qd.put(key='id', value=2, rel=QueryOpt.REL_AND)
    # ql.put(qd)
    # qd = QueryDict()
    # qd.put(key='name', opt=QueryOpt.OPT_LIKE, value='张%', rel=QueryOpt.REL_OR)
    # ql.put(qd)
    # qd = QueryDict()
    # qd.put(key='age', opt=QueryOpt.OPT_IS, value=None, rel=QueryOpt.REL_AND)
    # ql.put(qd)
    # querykw = {}
    # querykw.setdefault('tablename', 'student')
    # querykw.setdefault('query_list', ql)
    # rows = helper.query(**querykw)
    # print(rows)

    # 查询数量
    # querykw = {}
    # ql = QueryDictList()
    # qd = QueryDict()
    # qd.put(key='age', opt=QueryOpt.OPT_GTE, value=18)
    # ql.append(qd)
    # querykw.setdefault('tablename', 'student')
    # querykw.setdefault('query_list', ql)
    # count = helper.query_count(**querykw)
    # print(count)

    # sql添加
    # querykw = {}
    # querykw.setdefault('sql', 'insert into student (name, age) values ("张六", 19)')
    # print(helper.insert(**querykw))

    # # 指定表名和字段添加 (可批量插入)
    # querykw = {}
    # querykw.setdefault('tablename', 'student')
    # ql_list = []
    # ql = QueryDictList()
    # ql.append(QueryDict(key='name', value='赵七'))
    # ql.append(QueryDict(key='age', value=20))
    # ql_list.append(ql)
    # ql = QueryDictList()
    # ql.append(QueryDict(key='name', value='李八'))
    # ql.append(QueryDict(key='age', value=19))
    # ql_list.append(ql)
    # # 若单条添加，则添加一个ql，键值为query_list
    # querykw.setdefault('query_list', ql)
    # # 若批量添加,则添加一个ql_list,键值为ql_list
    # querykw.setdefault('ql_list', ql_list)
    # print(helper.insert(**querykw))

    # # sql更新
    # querykw = {}
    # querykw.setdefault('sql', 'update student set name=%s where id=1')
    # querykw.setdefault('params', ('张三',))
    # print(helper.update(**querykw))

    # # 指定表名和字段更新
    # querykw = {}
    # querykw.setdefault('tablename', 'student')
    # ul = QueryDictList()
    # ul.append(QueryDict(key='name', value='张三'))
    # ql = QueryDictList()
    # ql.append(QueryDict(key='id', value=1))
    # querykw.setdefault('update_list', ul)
    # querykw.setdefault('query_list', ql)
    # print(helper.update(**querykw))

    # # sql删除
    # querykw = {}
    # querykw.setdefault('sql', 'delete from student where id=%s')
    # querykw.setdefault('params', (7,))
    # print(helper.delete(**querykw))

    # # 指定表名和字段删除
    # querykw = {}
    # querykw.setdefault('tablename', 'student')
    # ql = QueryDictList()
    # ql.append(QueryDict(key='id', value=7))
    # querykw.setdefault('query_list', ql)
    # print(helper.delete(**querykw))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="fengluB",
    version="0.0.03_alpha2",
    # version="0.0.03_alpha3",
    author="Fenglu Niu",
    author_email="niufenglu@gmail.com",
    description="牛逢路的开发工具包",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ccooder/fengluU",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        "mysql-connector-python==8.0.11",
    ],
)
# 1. python setup.py sdist bdist_wheel
# python setup.py install bdist_wheetl
# 2. twine upload --repository-url https://test.pypi.org/legacy/ dist/*
# 2. twine upload dist/*
# 3. sudo pip install --index-url https://test.pypi.org/project/fengluA projectname
